@@ -45,12 +45,13 @@
 #include "backends/meta-backend-private.h"
 #include "meta-cursor-renderer-native.h"
 #include "meta-idle-monitor-native.h"
+#include "meta-renderer-native.h"
+
+#define DRM_CARD_UDEV_DEVICE_TYPE "drm_minor"
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GUdevDevice, g_object_unref)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GUdevClient, g_object_unref)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GUdevEnumerator, g_object_unref)
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(Login1Session, g_object_unref)
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(Login1Seat, g_object_unref)
 
 struct _MetaLauncher
 {
@@ -58,6 +59,8 @@ struct _MetaLauncher
   Login1Seat *seat_proxy;
 
   gboolean session_active;
+
+  int kms_fd;
 };
 
 static Login1Session *
@@ -108,21 +111,20 @@ get_seat_proxy (GCancellable *cancellable,
 static void
 session_unpause (void)
 {
-  ClutterBackend *clutter_backend;
-  CoglContext *cogl_context;
-  CoglDisplay *cogl_display;
+  MetaBackend *backend;
+  MetaRenderer *renderer;
 
-  clutter_backend = clutter_get_default_backend ();
-  cogl_context = clutter_backend_get_cogl_context (clutter_backend);
-  cogl_display = cogl_context_get_display (cogl_context);
-  cogl_kms_display_queue_modes_reset (cogl_display);
+  backend = meta_get_backend ();
+  renderer = meta_backend_get_renderer (backend);
+  meta_renderer_native_queue_modes_reset (META_RENDERER_NATIVE (renderer));
 
   clutter_evdev_reclaim_devices ();
   clutter_egl_thaw_master_clock ();
 
   {
     MetaBackend *backend = meta_get_backend ();
-    MetaCursorRenderer *renderer = meta_backend_get_cursor_renderer (backend);
+    MetaCursorRendererNative *cursor_renderer_native =
+      META_CURSOR_RENDERER_NATIVE (meta_backend_get_cursor_renderer (backend));
     ClutterActor *stage = meta_backend_get_stage (backend);
 
     /* When we mode-switch back, we need to immediately queue a redraw
@@ -130,7 +132,7 @@ session_unpause (void)
      * update. */
 
     clutter_actor_queue_redraw (stage);
-    meta_cursor_renderer_native_force_update (META_CURSOR_RENDERER_NATIVE (renderer));
+    meta_cursor_renderer_native_force_update (cursor_renderer_native);
     meta_idle_monitor_native_reset_idletime (meta_idle_monitor_get_core ());
   }
 }
@@ -282,6 +284,55 @@ on_active_changed (Login1Session *session,
   sync_active (self);
 }
 
+static guint
+count_devices_with_connectors (const gchar *seat_name,
+                               GList       *devices)
+{
+  g_autoptr (GHashTable) cards = NULL;
+  GList *tmp;
+
+  cards = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, g_object_unref);
+  for (tmp = devices; tmp != NULL; tmp = tmp->next)
+    {
+      GUdevDevice *device = tmp->data;
+      g_autoptr (GUdevDevice) parent_device = NULL;
+      const gchar *parent_device_type = NULL;
+      const gchar *parent_device_name = NULL;
+      const gchar *card_seat;
+
+      /* filter out the real card devices, we only care about the connectors */
+      if (g_udev_device_get_device_type (device) != G_UDEV_DEVICE_TYPE_NONE)
+        continue;
+
+      /* only connectors have a modes attribute */
+      if (!g_udev_device_has_sysfs_attr (device, "modes"))
+        continue;
+
+      parent_device = g_udev_device_get_parent (device);
+
+      if (g_udev_device_get_device_type (parent_device) == G_UDEV_DEVICE_TYPE_CHAR)
+        parent_device_type = g_udev_device_get_property (parent_device, "DEVTYPE");
+
+      if (g_strcmp0 (parent_device_type, DRM_CARD_UDEV_DEVICE_TYPE) != 0)
+        continue;
+
+      card_seat = g_udev_device_get_property (parent_device, "ID_SEAT");
+
+      if (!card_seat)
+        card_seat = "seat0";
+
+      if (g_strcmp0 (seat_name, card_seat) != 0)
+        continue;
+
+      parent_device_name = g_udev_device_get_name (parent_device);
+      g_hash_table_insert (cards,
+                           (gpointer) parent_device_name ,
+                           g_steal_pointer (&parent_device));
+    }
+
+  return g_hash_table_size (cards);
+}
+
 static gchar *
 get_primary_gpu_path (const gchar *seat_name)
 {
@@ -295,22 +346,46 @@ get_primary_gpu_path (const gchar *seat_name)
   g_udev_enumerator_add_match_name (enumerator, "card*");
   g_udev_enumerator_add_match_tag (enumerator, "seat");
 
+  /* We need to explicitly match the subsystem for now.
+   * https://bugzilla.gnome.org/show_bug.cgi?id=773224
+   */
+  g_udev_enumerator_add_match_subsystem (enumerator, "drm");
+
   devices = g_udev_enumerator_execute (enumerator);
   if (!devices)
     goto out;
 
+  /* For now, fail on systems where some of the connectors
+   * are connected to secondary gpus.
+   *
+   * https://bugzilla.gnome.org/show_bug.cgi?id=771442
+   */
+  if (g_getenv ("MUTTER_ALLOW_HYBRID_GPUS") == NULL)
+    {
+      guint num_devices;
+
+      num_devices = count_devices_with_connectors (seat_name, devices);
+      if (num_devices != 1)
+        goto out;
+    }
+
   for (tmp = devices; tmp != NULL; tmp = tmp->next)
     {
       g_autoptr (GUdevDevice) platform_device = NULL;
       g_autoptr (GUdevDevice) pci_device = NULL;
       GUdevDevice *dev = tmp->data;
       gint boot_vga;
+      const gchar *device_type;
       const gchar *device_seat;
 
       /* filter out devices that are not character device, like card0-VGA-1 */
       if (g_udev_device_get_device_type (dev) != G_UDEV_DEVICE_TYPE_CHAR)
         continue;
 
+      device_type = g_udev_device_get_property (dev, "DEVTYPE");
+      if (g_strcmp0 (device_type, DRM_CARD_UDEV_DEVICE_TYPE) != 0)
+        continue;
+
       device_seat = g_udev_device_get_property (dev, "ID_SEAT");
       if (!device_seat)
         {
@@ -350,9 +425,9 @@ get_primary_gpu_path (const gchar *seat_name)
         }
     }
 
+out:
   g_list_free_full (devices, g_object_unref);
 
-out:
   return path;
 }
 
@@ -463,8 +538,8 @@ meta_launcher_new (GError **error)
   self->seat_proxy = g_object_ref (seat_proxy);
 
   self->session_active = TRUE;
+  self->kms_fd = kms_fd;
 
-  clutter_egl_set_kms_fd (kms_fd);
   clutter_evdev_set_device_callbacks (on_evdev_device_open,
                                       on_evdev_device_close,
                                       self);
@@ -504,3 +579,9 @@ meta_launcher_activate_vt (MetaLauncher  *launcher,
 {
   return login1_seat_call_switch_to_sync (launcher->seat_proxy, vt, NULL, error);
 }
+
+int
+meta_launcher_get_kms_fd (MetaLauncher *self)
+{
+  return self->kms_fd;
+}
